import {
  ClassType,
} from '@overflow/commons/core/type';

import { Annotation } from '@overflow/commons/core/reflect';

import {
  Assert,
  AnnotationUtils,
} from '@overflow/commons/core/util';

import {
  InstanceDefinition,
  InstanceDefinitionRegistry,
  InstanceNameGenerator,
} from '@overflow/commons/di/factory';

import {
  InstanceNameType,
  NotInjectableError,
} from '@overflow/commons/di/type';

import { ScopeMetadataResolver } from './scope_metadata_resolver';
import { InjectableAnnotation, QualifierAnnotation } from '@overflow/commons/di/decorators';


export class InstanceDefinitionReader {
  private instanceDefinitionRegistry: InstanceDefinitionRegistry;
  private instanceNameGenerator: InstanceNameGenerator = new InstanceNameGenerator();
  private scopeMetadataResolver: ScopeMetadataResolver = new ScopeMetadataResolver();

  public constructor(registry: InstanceDefinitionRegistry) {
    Assert.notNull(registry, 'InstanceDefinitionRegistry must not be null');
    this.instanceDefinitionRegistry = registry;
  }

  public registerInjectables(...injectables: ClassType[]): void {
    injectables.forEach(injectable => {
      this.registerInjectable(injectable, undefined);
    });
  }

  public registerInjectable(injectable: ClassType, name: InstanceNameType | undefined, ...qualifiers: QualifierAnnotation[]): void {
    if (!AnnotationUtils.hasAnnotation(injectable, InjectableAnnotation)) {
      throw new NotInjectableError();
    }

    let definition: InstanceDefinition = new InstanceDefinition(injectable);
    let scopeMeta = this.scopeMetadataResolver.resolveScopeMetadata(definition);
    definition.scope = scopeMeta.scope;
    let instanceName =
                      undefined !== name
                      ? name
                      : this.instanceNameGenerator.generateInstanceName(definition, this.instanceDefinitionRegistry);

    if (undefined !== qualifiers) {
      qualifiers.forEach(qualifier => {
        definition.addQualifier(qualifier);
      });
    }

    this.instanceDefinitionRegistry.registerInstanceDefinition(instanceName, definition);
  }
}
