import {
  ClassType,
  TypeUtil,
} from '@overflow/commons/core/type';

import {AnnotatedElement} from './annotated_element';
import {
  Annotation,
} from './annotation';

export abstract class AccessibleObject implements AnnotatedElement {
  private _annotations: Map<ClassType, Annotation>;

  protected constructor() {
    this._annotations = new Map();
  }

  public _addAnnotation<AnnotationType extends Annotation>(annotation: AnnotationType): void {
    this._annotations.set(TypeUtil.getClass(annotation), annotation);
  }

  public isAnnotationPresent<AnnotationType extends Annotation>(annotationClass: ClassType<AnnotationType>): boolean {
    return null !== this.getAnnotation(annotationClass);
  }

  public getOwnAnnotation<AnnotationType extends Annotation>(annotationClass: ClassType<AnnotationType>): AnnotationType | undefined {
    return <AnnotationType>this._annotations.get(annotationClass);
  }

  public getOwnAnnotations(): Map<ClassType, Annotation> {
    return this._annotations;
  }

  public getAnnotation<AnnotationType extends Annotation>(annotationClass: ClassType<AnnotationType>): AnnotationType | undefined {
    return this.getOwnAnnotation(annotationClass);
  }

  public getAnnotations(): Map<ClassType, Annotation> {
    return this.getOwnAnnotations();
  }

}
