import {
  Class,
} from '@overflow/commons/core/reflect';

import {
  ClassType,
} from '@overflow/commons/core/type';

import { Assert } from '@overflow/commons/core/util';

import {
  QualifierAnnotation,
} from '@overflow/commons/di/decorators';

import {
  ScopeType,
  InjectModeType,
  NotInjectableError,
} from '@overflow/commons/di/type';

export class InstanceDefinition {
  private readonly attributes: Map<string, any> = new Map();
  private _instanceClass: Class;
  private _scope: ScopeType = ScopeType.Default;
  private _injectMode: InjectModeType = InjectModeType.No;


  public constructor(instanceClassType: ClassType) {
    let clazz = Class.forClassType(instanceClassType);
    if (undefined === clazz) {
      throw new NotInjectableError();
    }

    this._instanceClass = clazz;
  }

  public get instanceClass(): Class {
    return this._instanceClass;
  }

  // Metadata
  public getMetadata(): AnnotationMetadata {
    return undefined;
  }

  public set scope(scope: ScopeType) {
    this.scope = scope;
  }

  public get scope(): ScopeType {
    return this.scope;
  }

  public get injectMode(): InjectModeType {
    return this._injectMode;
  }

  public set injectMode(injectMode: InjectModeType) {
    this._injectMode = injectMode;
  }

  public isSingleton(): boolean {
    return ScopeType.Singleton === this.scope || ScopeType.Default === this.scope;
  }

  public isTransiant(): boolean {
    return ScopeType.Transiant === this.scope;
  }

  public addQualifier(qualifier: QualifierAnnotation): void {
    //
  }
}
